import 'mapping_crc.dart';
import 'simple_crc.dart';

abstract class CRC {
  /// 名称
  final String name;

  /// 宽度，最大支持 64 位
  final int width;

  /// 多项式
  final int poly;

  /// 初始值
  final int init;

  /// 输入反转
  final bool refIn;

  /// 输出反转
  final bool refOut;

  /// 输出异或值
  final int xorOut;

  /// 构造函数
  CRC(this.name, this.width, this.poly, this.init, this.refIn, this.refOut,
      this.xorOut);

  /// 使用指定参数创建 CRC 实例
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.create(String name, int width, int poly, int init, bool refIn,
      bool refOut, int xorOut,
      {bool mapping = false}) {
    final crc = mapping
        ? MappingCRC(name, width, poly, init, refIn, refOut, xorOut)
        : SimpleCRC(name, width, poly, init, refIn, refOut, xorOut);
    return crc;
  }

  /// CRC-4/ITU
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc4ITU({bool mapping = false}) {
    return CRC.create('CRC-4/ITU', 4, 0x03, 0x00, true, true, 0x00,
        mapping: mapping);
  }

  /// CRC-5/EPC
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc5EPC({bool mapping = false}) {
    return CRC.create('CRC-5/EPC', 5, 0x09, 0x09, false, false, 0x00,
        mapping: mapping);
  }

  /// CRC-5/ITU
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc5ITU({bool mapping = false}) {
    return CRC.create('CRC-5/ITU', 5, 0X15, 0x00, true, true, 0x00,
        mapping: mapping);
  }

  /// CRC-5/USB
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc5USB({bool mapping = false}) {
    return CRC.create('CRC-5/USB', 5, 0x05, 0x1F, true, true, 0x1F,
        mapping: mapping);
  }

  /// CRC-6/ITU
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc6ITU({bool mapping = false}) {
    return CRC.create('CRC-6/ITU', 6, 0x03, 0x00, true, true, 0x00,
        mapping: mapping);
  }

  /// CRC-7/MMC
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc7MMC({bool mapping = false}) {
    return CRC.create('CRC-7/MMC', 7, 0x09, 0x00, false, false, 0x00,
        mapping: mapping);
  }

  /// CRC-8
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc8({bool mapping = false}) {
    return CRC.create('CRC-8', 8, 0x07, 0x00, false, false, 0x00,
        mapping: mapping);
  }

  /// CRC-8/ITU
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc8ITU({bool mapping = false}) {
    return CRC.create('CRC-8/ITU', 8, 0x07, 0x00, false, false, 0x55,
        mapping: mapping);
  }

  /// CRC-8/MAXIM
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc8MAXIM({bool mapping = false}) {
    return CRC.create('CRC-8/MAXIM', 8, 0x31, 0x00, true, true, 0x00,
        mapping: mapping);
  }

  /// CRC-8/ROHC
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc8ROHC({bool mapping = false}) {
    return CRC.create('CRC-8/ROHC', 8, 0x07, 0xFF, true, true, 0x00,
        mapping: mapping);
  }

  /// CRC-16
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16({bool mapping = false}) {
    return CRC.create('CRC-16', 16, 0x8005, 0x0000, true, true, 0x0000,
        mapping: mapping);
  }

  /// CRC-16/CCITT
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16CCITT({bool mapping = false}) {
    return CRC.create('CRC-16/CCITT', 16, 0x1021, 0x0000, true, true, 0x0000,
        mapping: mapping);
  }

  /// CRC-16/CCITT-FALSE
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16CCITTFALSE({bool mapping = false}) {
    return CRC.create(
        'CRC-16/CCITT-FALSE', 16, 0x1021, 0xFFFF, false, false, 0x0000,
        mapping: mapping);
  }

  /// CRC-16/DNP
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16DNP({bool mapping = false}) {
    return CRC.create('CRC-16/DNP', 16, 0x3D65, 0x0000, true, true, 0xFFFF,
        mapping: mapping);
  }

  /// CRC-16/MAXIM
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16MAXIM({bool mapping = false}) {
    return CRC.create('CRC-16/MAXIM', 16, 0x8005, 0x0000, true, true, 0xFFFF,
        mapping: mapping);
  }

  /// CRC-16/MODBUS
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16MODBUS({bool mapping = false}) {
    return CRC.create('CRC-16/MODBUS', 16, 0x8005, 0xFFFF, true, true, 0x0000,
        mapping: mapping);
  }

  /// CRC-16/USB
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16USB({bool mapping = false}) {
    return CRC.create('CRC-16/USB', 16, 0x8005, 0xFFFF, true, true, 0xFFFF,
        mapping: mapping);
  }

  /// CRC-16/X25
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16X25({bool mapping = false}) {
    return CRC.create('CRC-16/X25', 16, 0x1021, 0xFFFF, true, true, 0xFFFF,
        mapping: mapping);
  }

  /// CRC-16/XMODEM
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc16XMODEM({bool mapping = false}) {
    return CRC.create('CRC-16/XMODEM', 16, 0x1021, 0x0000, false, false, 0x0000,
        mapping: mapping);
  }

  /// CRC-32
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc32({bool mapping = false}) {
    return CRC.create(
        'CRC-32', 32, 0x04C11DB7, 0xFFFFFFFF, true, true, 0xFFFFFFFF,
        mapping: mapping);
  }

  /// CRC-32/MPEG-2
  ///
  /// [mapping] - 指定是否使用查表法进行计算
  factory CRC.crc32MPEG2({bool mapping = false}) {
    return CRC.create(
        'CRC-32/MPEG-2', 32, 0x04C11DB7, 0xFFFFFFFF, false, false, 0x00000000,
        mapping: mapping);
  }

  int calculate(List<int> data);

  bool verify(List<int> data, int crc) {
    final expected = this.calculate(data);
    return crc == expected;
  }
}
